#ifndef _DECODER_H
#define _DECODER_H


/// siso memory structure to ease parallel windowed decoding
typedef struct siso_mem {
  int alpha_mem[W+1][8];      /// alpha memory
  int beta_local_mem[NWP][8]; /// beta local memory
  int beta[2][NS];         /// beta values
  int alpha_ext_mem[NS];   /// alpha_ext_mem values
  int beta_ext_mem[NS];    /// beta_ext_mem values
} siso_mem_t;

/// correction table computation
///\param **ic is the correction table pointer address
///\param nbit is the number of fractional bits
///\return the correction table size
int compute_table(int **ic, const int nbit); 
/// decoder side interleaver function
///\param K is the block size
///\param *perm is the permutation array
///\param *src is the array with data to permute
///\param *dst is the array with permuted data
void scrambler(int *perm, int *src, int *dst);
/// decoder side deinterleaver function
///\param K is the block size
///\param *perm is the permutation array
///\param *src is the array with data to permute
///\param *dst is the array with permuted data
void descrambler(int *perm, int *src, int *dst);
/// Parallel, in-order siso function
///\param NS is the number of states
///\param *llro is array of LLR exchanged by siso1 with siso2
///\param *llri is the intrinsic information array: u p
///\param SM_INIT is the state metric initialization value (-intinity approximation)
///\param P is the parallelism degree
///\param W is the window length
///\param NWP is the number of windows per siso
///\param *ptr is the array of pointers to siso memory structures
///\param EXT_MAX_VAL is the extrinsic information maximum value
///\param EXT_MIN_VAL is the extrinsic information minimum value
///\param *dec is the array with the decision information
///\param *ic is the correction table for the log-map
///\param ic_size is the correction table size
///\param *beta_init is the last trellis section initialization
void SISO1(int flag, int *llro, const int *llri, const int SM_INIT, siso_mem_t *ptr, int *dec, int *ic, int ic_size, int *beta_init);

#endif
