#ifndef _PARAMS_H
#define _PARAMS_H

/// input parameter function: reads from a file the configuration for current simulation
///\param *fp is the file pointer to the configuration file 
///\param *K is the block size 
///\param *max_frame is the maximum number of simulated frames
///\param *oname is the output (ber/fer results) file name
///\param *lfsr_seed is the seed of the lfsr used to randomly generate uncoded bits
///\param *unif_seed is the seed of the uniform generator used to produce gaussian noise samples
///\param *code is the type of code choosen: UMTS (0), LTE (1)
///\param *max_point is the maximum number of SNR points simulated
///\param *ebn0_min is the minimum simulated SNR value
///\param *ebn0_step is the SNR step from one point to the successive
///\param *NBF_INT is the number of fractional bits to represent the intrinsic information
///\param *NBI_INT is the number of integer bits to represent the intrinsic informtaion
///\param *INT_MAX_VAL is the maximum value to represent intrinsic information
///\param *INT_MIN_VAL is the minimum value to represent intrinsic information
///\param *EXT_MAX_VAL is the maximum value to represent extrinsic information
///\param *EXT_MIN_VAL is the minimum value to represent extrinsic information
///\param *min_iter is the minimum number of decoding iterations 
///\param *max_iter is the maximum number of decoding iterations 
///\param *NB_SM is the number of bits used to represent the state metrics
///\param *P is the decoder parallelism degree
///\param *W is the window length
///\param *max_err is the maximum number of wrong frames counted
///\param *time_check is the minimum amount of elapsed time to save results obtained so far
///\param *f is the starting frame number (for resuming simulation)
///\param *max_point_comp is the maximum simulation point computed so far
///\param *min_point minimum current starting point
///\param **tot_frames is the total number of simulated frame array
///\param ***frame_err is the frame error counter array
///\param ***bit_err is the bit error counter array
void get_params( int *K, int *max_frame, int *NBF_INT, int *EXT_MIN_VAL, int *EXT_MAX_VAL, int *min_iter, int *max_iter, int *NB_SM, int *P, int *W, int *f);

/// show parameter function: print on screen the configuration for current simulation
///\param K is the uncoded block size 
///\param N is the coded block size 
///\param P is the decoder parallelism degree
///\param W is the window length
///\param NWP is the number of windows per siso
///\param max_frame is the maximum number of simulated frames
///\param min_iter is the minimum number of decoding iterations 
///\param max_iter is the maximum number of decoding iterations 
///\param max_err is the maximum number of wrong frames counted
///\param ebn0_min is the minimum simulated SNR value
///\param ebn0_step is the SNR step from one point to the successive
///\param max_point is the maximum number of SNR points simulated
///\param NBI_INT is the number of integer bits to represent the intrinsic information
///\param NBF_INT is the number of fractional bits to represent the intrinsic information
///\param INT_MAX_VAL is the maximum value to represent intrinsic information
///\param INT_MIN_VAL is the minimum value to represent intrinsic information
///\param EXT_MAX_VAL is the maximum value to represent extrinsic information
///\param EXT_MIN_VAL is the minimum value to represent extrinsic information
///\param lfsr_seed is the seed of the lfsr used to randomly generate uncoded bits
///\param unif_seed is the seed of the uniform generator used to produce gaussian noise samples
///\param time_check is the minimum amount of elapsed time to save results obtained so far
///\param code is the type of code used: 0 -> UMTS, 1 -> LTE
///\param NB_SM is the number of bits to inizialize the state metrics
///\param ic_size is the Log-MAP correction table size
///\param *ic is the Log-MAP correction table
void show_params(int K, int N, int P, int W, int NWP, int max_frame, int min_iter, int max_iter, int max_err, double ebn0_min, double ebn0_step, int max_point, int NBI_INT, int NBF_INT, int INT_MAX_VAL, int INT_MIN_VAL, int EXT_MAX_VAL, int EXT_MIN_VAL, unsigned int lfsr_seed, unsigned int unif_seed, int time_check, int code, int NB_SM, int ic_size, int *ic);

#endif
