#ifndef _RESULTS_H
#define _RESULTS_H

/// measure the ber and the fer
///\param K is the block size
///\param *dec is the hard decision couple
///\param *frame is the original uncoded frame
///\param **bit_err is the array where bit error counters are stored
///\param **frame_err is the array where frame error counters are stored
///\param itc is the current iteration
///\param curr_point is the current SNR point
///\return the number of error counted for current frame
int BER_meter(int K, int *dec, int *frame, int **bit_err, int **frame_err, int itc, int curr_point);
/// stores the simulation results (ber/fer)
///\param *f_BER is the results file pointer
///\param min_iter is the minimun number of iterations
///\param max_iter is the maximum number of iterations
///\param max_point_comp is the maximum SNR point computed so far
///\param ebn0_min is the minimum SNR value simulated
///\param ebn0_step is the SNR step between consecutive SNR points
///\param **bit_err is the array where bit error counters are stored
///\param **frame_err is the array where frame error counters are stored
///\param *tot_frames is the array where the total number of frames simulated per each SNR point is stored
///\param K is the frame size
void store_results(int min_iter, int max_iter, int max_point_comp, double ebn0_min, double ebn0_step, int **bit_err, int **frame_err, int *tot_frames, int K);

///\param K is the block size 
///\param max_frame is the maximum number of simulated frames
///\param *oname is the output (ber/fer results) file name
///\param lfsr_seed is the seed of the lfsr used to randomly generate uncoded bits
///\param unif_seed is the seed of the uniform generator used to produce gaussian noise samples
///\param code is the type of code choosen: UMTS (0), LTE (1)
///\param max_point is the maximum number of SNR points simulated
///\param ebn0_min is the minimum simulated SNR value
///\param ebn0_step is the SNR step from one point to the successive
///\param NBF_INT is the number of fractional bits to represent the intrinsic information
///\param NBI_INT is the number of integer bits to represent the intrinsic informtaion
///\param INT_MAX_VAL is the maximum value to represent intrinsic information
///\param INT_MIN_VAL is the minimum value to represent intrinsic information
///\param EXT_MAX_VAL is the maximum value to represent extrinsic information
///\param EXT_MIN_VAL is the minimum value to represent extrinsic information
///\param min_iter is the minimum number of decoding iterations 
///\param max_iter is the maximum number of decoding iterations 
///\param NB_SM is the number of bits used to represent the state metrics
///\param P is the decoder parallelism degree
///\param W is the window length
///\param max_err is the maximum number of wrong frames counted
///\param time_check is the minimum amount of elapsed time to save results obtained so far
///\param f is the starting frame number (for resuming simulation)
///\param max_point_comp is the maximum simulation point computed so far
///\param min_point minimum current starting point
///\param *tot_frames is the total number of simulated frame array
///\param **frame_err is the frame error counter array
///\param **bit_err is the bit error counter array
void save_simulation(int K, int max_frame, unsigned int lfsr_seed, unsigned int unif_seed, int code, int max_point, double ebn0_min, double ebn0_step, int NBF_INT, int NBI_INT, int INT_MAX_VAL, int INT_MIN_VAL, int EXT_MIN_VAL, int EXT_MAX_VAL, int min_iter, int max_iter, int NB_SM, int P, int W, int max_err, int time_check, int f, int max_point_comp, int min_point, int *tot_frames, int **frame_err, int **bit_err);

#endif
