#include<stdio.h>
#include<stdlib.h>
#include"params.h"
#include"myservs.h"

/// input parameter function: reads from a file the configuration for current simulation
///\param *fp is the file pointer to the configuration file 
///\param *K is the block size 
///\param *max_frame is the maximum number of simulated frames
///\param *oname is the output (ber/fer results) file name
///\param *lfsr_seed is the seed of the lfsr used to randomly generate uncoded bits
///\param *unif_seed is the seed of the uniform generator used to produce gaussian noise samples
///\param *code is the type of code choosen: UMTS (0), LTE (1)
///\param *max_point is the maximum number of SNR points simulated
///\param *ebn0_min is the minimum simulated SNR value
///\param *ebn0_step is the SNR step from one point to the successive
///\param *NBF_INT is the number of fractional bits to represent the intrinsic information
///\param *NBI_INT is the number of integer bits to represent the intrinsic informtaion
///\param *INT_MAX_VAL is the maximum value to represent intrinsic information
///\param *INT_MIN_VAL is the minimum value to represent intrinsic information
///\param *EXT_MAX_VAL is the maximum value to represent extrinsic information
///\param *EXT_MIN_VAL is the minimum value to represent extrinsic information
///\param *min_iter is the minimum number of decoding iterations 
///\param *max_iter is the maximum number of decoding iterations 
///\param *NB_SM is the number of bits used to represent the state metrics
///\param *P is the decoder parallelism degree
///\param *W is the window length
///\param *max_err is the maximum number of wrong frames counted
///\param *time_check is the minimum amount of elapsed time to save results obtained so far
///\param *f is the starting frame number (for resuming simulation)
///\param *max_point_comp is the maximum simulation point computed so far
///\param *min_point minimum current starting point
///\param **tot_frames is the total number of simulated frame array
///\param ***frame_err is the frame error counter array
///\param ***bit_err is the bit error counter array
void get_params( int *K, int *max_frame, int *NBF_INT, int *EXT_MIN_VAL, int *EXT_MAX_VAL, int *min_iter, int *max_iter, int *NB_SM, int *P, int *W, int *f)
{
  char stringa[LARGE_INT_VAL];
  int tmp;
  int i,j;

  /*************************** frame size and code choice*************/

  *K=40;

  /*************************** parallelism degree *************/

  *P=1;
  /*************************** window size *************/

  *W=1;
  /*************************** Max number of iterations *************/

  *min_iter=1;
  *max_iter=4;
  /**************************  Max number of frame simulated *************/

  *max_frame=1;

  /************************** Intrinsic information *****************************/
  /********* number of fractional bits, maximum and minimum value ****************/
  *NBF_INT=1;

  /************************** Extrinsic information *****************************/
  /*********************** maximum and minimum value **************************/
  *EXT_MAX_VAL=127;
  *EXT_MIN_VAL=-128;          
  /************************** State metrics *****************************/
  /*********************** number of bits **************************/
  *NB_SM=12;             
  /*********************** Starting frame number (for resuming simulation) ***************/
  *f=1;                
}

/// show parameter function: print on screen the configuration for current simulation
///\param K is the uncoded block size 
///\param N is the coded block size 
///\param P is the decoder parallelism degree
///\param W is the window length
///\param NWP is the number of windows per siso
///\param max_frame is the maximum number of simulated frames
///\param min_iter is the minimum number of decoding iterations 
///\param max_iter is the maximum number of decoding iterations 
///\param max_err is the maximum number of wrong frames counted
///\param ebn0_min is the minimum simulated SNR value
///\param ebn0_step is the SNR step from one point to the successive
///\param max_point is the maximum number of SNR points simulated
///\param NBI_INT is the number of integer bits to represent the intrinsic information
///\param NBF_INT is the number of fractional bits to represent the intrinsic information
///\param INT_MAX_VAL is the maximum value to represent intrinsic information
///\param INT_MIN_VAL is the minimum value to represent intrinsic information
///\param EXT_MAX_VAL is the maximum value to represent extrinsic information
///\param EXT_MIN_VAL is the minimum value to represent extrinsic information
///\param lfsr_seed is the seed of the lfsr used to randomly generate uncoded bits
///\param unif_seed is the seed of the uniform generator used to produce gaussian noise samples
///\param time_check is the minimum amount of elapsed time to save results obtained so far
///\param code is the type of code used: 0 -> UMTS, 1 -> LTE
///\param NB_SM is the number of bits to inizialize the state metrics
///\param ic_size is the Log-MAP correction table size
///\param *ic is the Log-MAP correction table
void show_params(int K, int N, int P, int W, int NWP, int max_frame, int min_iter, int max_iter, int max_err, double ebn0_min, double ebn0_step, int max_point, int NBI_INT, int NBF_INT, int INT_MAX_VAL, int INT_MIN_VAL, int EXT_MAX_VAL, int EXT_MIN_VAL, unsigned int lfsr_seed, unsigned int unif_seed, int time_check, int code, int NB_SM, int ic_size, int *ic)
{
  int i;

  printf("Simulation parameters:\n");
  printf("K=%d", K);
  if (code == 0)
    printf("\tUMTS\n");
  else
    printf("\tLTE\n");
  printf("N=%d\n", N);
  printf("P=%d, W=%d, NWP=%d\n", P, W, NWP);
  printf("Simulating %d frames and max %d wrong frames\n", max_frame, max_err);
  printf("Simulating %d points from %lf with step %lf\n", max_point, ebn0_min, ebn0_step);
  printf("Simulating %d - %d iterations\n", min_iter, max_iter);
  printf("Intrinsic information: %d integer bits, %d fractional bits, min=%d - max=%d\n", NBI_INT, NBF_INT, INT_MIN_VAL, INT_MAX_VAL);
  printf("Extrinsic information: min=%d - max=%d\n", EXT_MIN_VAL, EXT_MAX_VAL);
  printf("State metrics initialization on %d bits\n", NB_SM);
  printf("Using Log-MAP approximation: table size is %d\n", ic_size);
  printf("Table content:");
  for (i=0; i<ic_size; i++)
    printf(" %d", ic[i]);
  printf("\n");
  printf("Pseudo-random generator seeds: lfsr=%u, unif=%u\n", lfsr_seed, unif_seed);
  printf("Simulation checked every %d seconds\n", time_check);
}

