#include<stdio.h>
#include<stdlib.h>

#include"results.h"
#include"myservs.h"

/// measure the ber and the fer
///\param K is the block size
///\param *dec is the hard decision couple
///\param *frame is the original uncoded frame
///\param **bit_err is the array where bit error counters are stored
///\param **frame_err is the array where frame error counters are stored
///\param itc is the current iteration
///\param curr_point is the current SNR point
///\return the number of error counted for current frame
int BER_meter(int K, int *dec, int *frame, int **bit_err, int **frame_err, int itc, int curr_point)
{
  int i;
  int bit_dec;
  int count;

  /// error counter  
  count=0;
  for (i=0; i<K; i++)
  {
    if (dec[i] >= 0)
      bit_dec = 1;
    else
      bit_dec = 0;
    if (bit_dec != frame[i])
      count++;
  }

  bit_err[itc][curr_point] += count;
  if (count != 0)
    frame_err[itc][curr_point]++;
  return count;
}

/// stores the simulation results (ber/fer)
///\param *f_BER is the results file pointer
///\param min_iter is the minimun number of iterations
///\param max_iter is the maximum number of iterations
///\param max_point_comp is the maximum SNR point computed so far
///\param ebn0_min is the minimum SNR value simulated
///\param ebn0_step is the SNR step between consecutive SNR points
///\param **bit_err is the array where bit error counters are stored
///\param **frame_err is the array where frame error counters are stored
///\param *tot_frames is the array where the total number of frames simulated per each SNR point is stored
///\param K is the frame size
void store_results( int min_iter, int max_iter, int max_point_comp, double ebn0_min, double ebn0_step, int **bit_err, int **frame_err, int *tot_frames, int K)
{
  int i,j;

  //rewind(f_BER);

  printf("********* BER Results **********\n");
  printf("       ");
  //  for(i=min_iter-1; i<max_iter; i+=min_iter)
  for(i=min_iter-1; i<max_iter; i++)
  {
    printf(" %8d|", i+1);
  }
  printf("\n");

  for(j=0; j<max_point_comp+1; j++)
  {
    printf("%6.2f|", ebn0_min+j*ebn0_step);
    //    for(i=min_iter-1; i<max_iter; i+=min_iter)
    for(i=min_iter-1; i<max_iter; i++)
    {
      printf("%.2e|", (bit_err[i][j])/(double)tot_frames[j]/K);
    }
    printf("\n");
  }

  printf("\n");
  printf("********* FER Results **********\n");
  printf("       ");
  //  for(i=min_iter-1; i<max_iter; i+=min_iter)
  for(i=min_iter-1; i<max_iter; i++)
  {
    printf("%8d|", i+1);
  }
  printf("\n");
  for(j=0; j<max_point_comp+1; j++)
  {
    printf("%6.2f|", ebn0_min+j*ebn0_step);
    //    for(i=min_iter-1; i<max_iter; i+=min_iter)
    for(i=min_iter-1; i<max_iter; i++)
    {
      printf( "%.2e|", (frame_err[i][j])/(double)tot_frames[j]);
    }
    printf("\n");
  }

  //fflush(f_BER);
  printf("Results stored.\n");
}

///\param K is the block size 
///\param max_frame is the maximum number of simulated frames
///\param *oname is the output (ber/fer results) file name
///\param lfsr_seed is the seed of the lfsr used to randomly generate uncoded bits
///\param unif_seed is the seed of the uniform generator used to produce gaussian noise samples
///\param code is the type of code choosen: UMTS (0), LTE (1)
///\param max_point is the maximum number of SNR points simulated
///\param ebn0_min is the minimum simulated SNR value
///\param ebn0_step is the SNR step from one point to the successive
///\param NBF_INT is the number of fractional bits to represent the intrinsic information
///\param NBI_INT is the number of integer bits to represent the intrinsic informtaion
///\param INT_MAX_VAL is the maximum value to represent intrinsic information
///\param INT_MIN_VAL is the minimum value to represent intrinsic information
///\param EXT_MAX_VAL is the maximum value to represent extrinsic information
///\param EXT_MIN_VAL is the minimum value to represent extrinsic information
///\param min_iter is the minimum number of decoding iterations 
///\param max_iter is the maximum number of decoding iterations 
///\param NB_SM is the number of bits used to represent the state metrics
///\param P is the decoder parallelism degree
///\param W is the window length
///\param max_err is the maximum number of wrong frames counted
///\param time_check is the minimum amount of elapsed time to save results obtained so far
///\param f is the starting frame number (for resuming simulation)
///\param max_point_comp is the maximum simulation point computed so far
///\param min_point minimum current starting point
///\param *tot_frames is the total number of simulated frame array
///\param **frame_err is the frame error counter array
///\param **bit_err is the bit error counter array
void save_simulation(int K, int max_frame, unsigned int lfsr_seed, unsigned int unif_seed, int code, int max_point, double ebn0_min, double ebn0_step, int NBF_INT, int NBI_INT, int INT_MAX_VAL, int INT_MIN_VAL, int EXT_MIN_VAL, int EXT_MAX_VAL, int min_iter, int max_iter, int NB_SM, int P, int W, int max_err, int time_check, int f, int max_point_comp, int min_point, int *tot_frames, int **frame_err, int **bit_err)
{
  int i,j;

  /************************ Check results period **********************************/  
  printf("Check results period\n");
  printf("%d\n", time_check);  
  /*************************** frame size and code choice*************/
  printf("Frame size and code choice\n");
  printf("%d,%d\n", K, code);
  /*************************** parallelism degree *************/
  printf("Parallelism degree\n");
  printf("%d\n", P);
  /*************************** window size *************/
  printf("Window size\n");
  printf("%d\n", W);
  /*************************** Max number of iterations *************/
  printf("Minimum-Maximum number of iterations\n");
  printf("%d,%d\n", min_iter, max_iter);
  /**************************  Max number of frame simulated *************/
  printf("Maximum number of frame simulated\n");
  printf("%d\n", max_frame);
  /**************************  Max number of wrong frames simulated *************/
  printf("Maximum number of wrong frames simulated\n");
  printf("%d\n", max_err);
  /************************** Eb/N0 *****************************/
  printf("Eb/N0: min, step, max_point\n");
  printf("%lf,%lf,%d\n", ebn0_min, ebn0_step, max_point);
  /************************** lfsr_seed *****************************/
  printf("LFSR seed\n");
  printf("%u\n", lfsr_seed);
  /************************** unif_seed *****************************/
  printf("Uniform generator seed\n");
  printf("%u\n", unif_seed);
  /************************** Intrinsic information *****************************/
  /********* number of fractional bits, maximum and minimum value ****************/
  printf("Intrinsic information: integer bits, fractional bits, max-min values\n");
  printf("%d,%d,%d,%d\n", NBI_INT, NBF_INT, INT_MAX_VAL, INT_MIN_VAL);  
  /************************** Extrinsic information *****************************/
  /*********************** maximum and minimum value **************************/
  printf("Extrinsic information: max-min values\n");
  printf("%d,%d\n", EXT_MAX_VAL, EXT_MIN_VAL);  
  /************************** State metrics *****************************/
  /*********************** number of bits **************************/
  printf("State metrics: number of bits\n");
  printf("%d\n", NB_SM);  
  /*********************** Starting frame number (for resuming simulation) ***************/
  printf("Starting frame\n");
  printf("%d\n", f);  
  /***************** Maximum point computed ************/
  printf("Maximum point computed\n");
  printf("%d\n", max_point_comp);
  /************************** Minimum point to compute ************/
  printf("Minimum point to compute\n");
  printf("%d\n", min_point);
  /***** Status of counters */
  printf("Status of counters\n");
  for(j=0; j<max_point; j++)
  {
    printf("%d\n", tot_frames[j]);
    for(i=0; i<max_iter; i++)
      printf("%d,", bit_err[i][j]);
    printf("\n");
    for(i=0; i<max_iter; i++)
      printf("%d,", frame_err[i][j]);
    printf("\n");

    //for(i=0; i<*max_iter; i++)
    //fscanf(fp,"%d,",&n_iter[i][j]);
    //fscanf(fp,"\n");
  }    
 // fclose(fp);
}
