#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"decoder.h"
#include"myservs.h"
#include "rt/rt_api.h"
/// correction table computation
///\param *ic is the correction table pointer address
///\param nbit is the number of fractional bits
///\return the correction table size

int compute_table(int **ic, const int nbit)
{
  int factor;
  int temp;
  int i;
  int size;

  factor = 1 << nbit;

  /// Find the table size
  size=-1;
  do
  {
    size += 1;
    temp = (int)(factor*log(1.+exp(-(float)(size)/(float)factor))+0.5);
  }while(temp!=0);

  *ic = (int*) myalloc(size, sizeof(int), 0, "compute_table", "*ic");

  /// Fill the table
  for(i=0; i<size; i++)
  {
    (*ic)[i]=(int)floor(factor*log(1.+exp(-(float)i/(float)factor))+0.5);
  }  

  return size;
}

void scrambler(int *perm, int *src, int *dst, int K)
{
  int i;

  for(i=0; i<K; i++)
    dst[i]=src[perm[i]];  
}

void descrambler(int *perm, int *src, int *dst, int K)
{
  int i;

  for(i=0; i<K; i++)
    dst[perm[i]] = src[i];  
}

/// 2 inputs log-map function
///\param a first input
///\param b second input
///\param *ic correction table pointer 
///\param ic_size correction table size
///\return the maximum
static int eopq2 (int a, int b, int *ic, int ic_size)
{
  register int ab;  
  register int val;

  ab = a-b;
  if (ab >= 0)
    val = a;
  else
  {
    val = b;
    ab = -ab;
  }

  if (ab < ic_size)
    return (val+ic[ab]);
  else
    return val;
}

/// 8 inputs log-map function
///\param a first input
///\param b second input
///\param c third input
///\param d fourth input
///\param e fifth input
///\param f sixth input
///\param g seventh input
///\param h eighth input
///\param *ic correction table pointer 
///\param ic_size correction table size
///\return the maximum
static int eopq8 (int a, int b, int c, int d, int e, int f, int g, int h, int *ic, int ic_size)
{
	int i=eopq2(eopq2(a,b,ic,ic_size),eopq2(c,d,ic,ic_size), ic, ic_size);
	int l=eopq2(eopq2(e,f,ic,ic_size),eopq2(g,h,ic,ic_size), ic, ic_size);
  return eopq2(i,l, ic, ic_size);
}

/// compares x and y and put the max in m1[idx] and the min in m2[idx]
///\param x first value
///\param y second value
///\param *m1 first max array
///\param *m2 second max array
static void max12_0(int x, int y, int *m1, int *m2, int idx)
{
  if (x > y)
  {
    m1[idx] = x;
    m2[idx] = y;
  }
  else
  {
    m1[idx] = y;
    m2[idx] = x;
  }
}

/// compute the first and second maximum using m1[k*idx], m1[k*idx+1], m2[k*idx] and m2[k*idx+1]
///\param *m1_in is the first maximum array (input)
///\param *m2_in is the second maximum array (input)
///\param *m1_out is the first maximum array (output)
///\param *m2_out is the second maximum array (output)
///\param idx is the position
///\param k is the displacement factor 
static void max12_1(int *m1_in, int *m2_in, int *m1_out, int *m2_out, int idx, int k)
{
  if (m1_in[k*idx] > m1_in[k*idx+1])
  {
    m1_out[idx] = m1_in[k*idx];
    if (m1_in[k*idx+1] > m2_in[k*idx])
      m2_out[idx] = m1_in[k*idx+1];
    else
      m2_out[idx] = m2_in[k*idx];
  }
  else
  {
    m1_out[idx] = m1_in[k*idx+1];
    if (m1_in[k*idx] > m2_in[k*idx+1])
      m2_out[idx] = m1_in[k*idx];
    else
      m2_out[idx] = m2_in[k*idx+1];
  }
}

/// 8 inputs log-map function (approximation proposed by Stelios)
///\param a first input
///\param b second input
///\param c third input
///\param d fourth input
///\param e fifth input
///\param f sixth input
///\param g seventh input
///\param h eighth input
///\param *ic correction table pointer 
///\param ic_size correction table size
///\return the maximum
static int eopq8_new(int a, int b, int c, int d, int e, int f, int g, int h, int *ic, int ic_size)
{
  int m1_0[4];
  int m2_0[4];

  int m1_1[2];
  int m2_1[2];

  int m1_2[1];
  int m2_2[1];

  register int ab;    

  /// find the first two maximum values
  max12_0(a, b, m1_0, m2_0, 0);
  max12_0(c, d, m1_0, m2_0, 1);
  max12_0(e, f, m1_0, m2_0, 2);
  max12_0(g, h, m1_0, m2_0, 3);

  max12_1(m1_0, m2_0, m1_1, m2_1, 0, 2);
  max12_1(m1_0, m2_0, m1_1, m2_1, 1, 2);

  max12_1(m1_1, m2_1, m1_2, m2_2, 0, 1);

  /// apply the correction factor
  ab = m1_2[0]-m2_2[0];
  if (ab < ic_size)
    return (m1_2[0]+ic[ab]);
  else
    return m1_2[0];
}

/// sliding window in-order siso function
///\param *llro is array of LLR exchanged by siso1 with siso2
///\param *llri is the intrinsic information array: u p
///\param W is the window length
///\param *ptr is the array of pointers to siso memory structures
///\param max_val is the extrinsic information maximum value
///\param min_val is the extrinsic information minimum value
///\param *dec is the array with the decision information
///\param *ic is the correction table for the log-map
///\param ic_size is the correction table size
///\return the position of the last new beta (0/1)
static int siso1_wind(int flag,int delta, int *llro, const int *llri, siso_mem_t *ptr, int *dec, int *ic, int ic_size, int W, int EXT_MAX_VAL, int EXT_MIN_VAL)
{
  int step;
  int apr;
  int intu, intp;
	if(flag==0)
		llri=llri+delta;

  const int zero = 0;

  /// label: U P
  int gamma01;
  int gamma02;
  int gamma03;

  int *alpha_old;
  int *alpha_new;

  int *beta_old;
  int *beta_new;
  
  int beta_idx;

  int x00;
  int x01;
  int x02;
  int x03;
  int x04;
  int x05;
  int x06;
  int x07;

  int x10;
  int x11;
  int x12;
  int x13;
  int x14;
  int x15;
  int x16;
  int x17;

  int *alpha;

  int lp0;
  int lp1;

  int apo;

  int ext;

  int tmp;

  /// forward recursion
  for (step=0; step<W; step++)
  {
   /// intrinsic
		if(flag==0)
		{
    	intu = llri[2*step];   /// u
    	intp = llri[2*step+1]; /// p
		}
		else
			intp = llri[step];

    /// a-priori
    apr = llro[step];   

    /// gamma
    gamma01 = zero + intp;
		if(flag==0)
		{
    	gamma02 = intu + zero + apr;
    	gamma03 = intu + intp + apr;
		}
		else
		{
			gamma02 = zero + apr;
    	gamma03 = intp + apr;
		}

    /// alpha
    alpha_old = ptr->alpha_mem[step];
    alpha_new = ptr->alpha_mem[step+1];

    alpha_new[0] = eopq2(alpha_old[0],         alpha_old[1]+gamma03, ic, ic_size);
    alpha_new[1] = eopq2(alpha_old[2]+gamma02, alpha_old[3]+gamma01, ic, ic_size);
    alpha_new[2] = eopq2(alpha_old[4]+gamma01, alpha_old[5]+gamma02, ic, ic_size);
    alpha_new[3] = eopq2(alpha_old[6]+gamma03, alpha_old[7],         ic, ic_size);
    alpha_new[4] = eopq2(alpha_old[0]+gamma03, alpha_old[1],         ic, ic_size);
    alpha_new[5] = eopq2(alpha_old[2]+gamma01, alpha_old[3]+gamma02, ic, ic_size);
    alpha_new[6] = eopq2(alpha_old[4]+gamma02, alpha_old[5]+gamma01, ic, ic_size);
    alpha_new[7] = eopq2(alpha_old[6],         alpha_old[7]+gamma03, ic, ic_size);
  }
 
  /// backward recursion and decoding
  beta_idx = 0;
  for (step=W-1; step>=0; step--)
  {
    if(flag==0)
		{
    	intu = llri[2*step];   /// u
    	intp = llri[2*step+1]; /// p
		}
		else
			intp = llri[step];

    /// a-priori
    apr = llro[step];   

    /// gamma
    gamma01 = zero + intp;
		if(flag==0)
		{
    	gamma02 = intu + zero + apr;
    	gamma03 = intu + intp + apr;
		}
		else
		{
			gamma02 = zero + apr;
    	gamma03 = intp + apr;
		}

    /// beta and decoding
    beta_old = ptr->beta[beta_idx];
    beta_new = ptr->beta[beta_idx^1];
    alpha = ptr->alpha_mem[step];

    x00 = beta_old[0]; 
    x01 = beta_old[4];
    x02 = beta_old[5] + gamma01;
    x03 = beta_old[1] + gamma01;
    x04 = beta_old[2] + gamma01;
    x05 = beta_old[6] + gamma01;
    x06 = beta_old[7];
    x07 = beta_old[3];

    x10 = beta_old[4] + gamma03; 
    x11 = beta_old[0] + gamma03;
    x12 = beta_old[1] + gamma02;
    x13 = beta_old[5] + gamma02;
    x14 = beta_old[6] + gamma02;
    x15 = beta_old[2] + gamma02;
    x16 = beta_old[3] + gamma03;
    x17 = beta_old[7] + gamma03;

    beta_new[0] = eopq2(x00, x10, ic, ic_size);
    beta_new[1] = eopq2(x01, x11, ic, ic_size);
    beta_new[2] = eopq2(x02, x12, ic, ic_size);
    beta_new[3] = eopq2(x03, x13, ic, ic_size);
    beta_new[4] = eopq2(x04, x14, ic, ic_size);
    beta_new[5] = eopq2(x05, x15, ic, ic_size);
    beta_new[6] = eopq2(x06, x16, ic, ic_size);
    beta_new[7] = eopq2(x07, x17, ic, ic_size);

    x00 += alpha[0];
    x01 += alpha[1];
    x02 += alpha[2];
    x03 += alpha[3];
    x04 += alpha[4];
    x05 += alpha[5];
    x06 += alpha[6];
    x07 += alpha[7];

    lp0 = eopq8(x00, x01, x02, x03, x04, x05, x06, x07, ic, ic_size);
    //lp0 = eopq8_new(x00, x01, x02, x03, x04, x05, x06, x07, ic, ic_size);

    x10 += alpha[0];
    x11 += alpha[1];
    x12 += alpha[2];
    x13 += alpha[3];
    x14 += alpha[4];
    x15 += alpha[5];
    x16 += alpha[6];
    x17 += alpha[7];

    lp1 = eopq8(x10, x11, x12, x13, x14, x15, x16, x17, ic, ic_size);
    //lp1 = eopq8_new(x10, x11, x12, x13, x14, x15, x16, x17, ic, ic_size);

    /// a-posteriori
    apo = lp1 - lp0;

/// extrinsic not saturated
		if(flag==0)
		{
    	ext = apo - apr - intu;
	    /// multiplication by 0.75, readding intrinsic and saturation
	    tmp = (ext >> 1) + (ext >> 2) + intu;
		}
		else
		{
			ext = apo - apr;
	    /// multiplication by 0.75, readding intrinsic and saturation
    	tmp = (ext >> 1) + (ext >> 2);
		}

    if (tmp > EXT_MAX_VAL)
      llro[step] = EXT_MAX_VAL;
    else if (tmp < EXT_MIN_VAL)
      llro[step] = EXT_MIN_VAL;
    else
      llro[step] = tmp;

    /// decision
    dec[step] = llro[step] + apr;
		
    beta_idx ^= 1;
  }

  return (beta_idx);
}

/// Parallel, in-order siso function
///\param NS is the number of states
///\param *llro is array of LLR exchanged by siso1 with siso2
///\param *llri is the intrinsic information array: u p
///\param SM_INIT initialization value for SM
///\param P is the parallelism degree
///\param W is the window length
///\param NWP is the number of windows per siso
///\param *ptr is the array of pointers to siso memory structures
///\param EXT_MAX_VAL is the extrinsic information maximum value
///\param EXT_MIN_VAL is the extrinsic information minimum value
///\param *dec is the array with the decision information
///\param *ic is the correction table for the log-map
///\param ic_size is the correction table size
///\param *beta_init is the last trellis section initialization
void SISO1(int flag, int NS, int *llro, const int *llri, const int SM_INIT, int P,int W, int NWP,int EXT_MAX_VAL, int EXT_MIN_VAL, siso_mem_t *ptr, int *dec, int *ic, int ic_size, int *beta_init)
{

  int siso_idx;
	int state_idx;
  int wind_idx;
  int delta;
  int beta_idx;

	if(rt_core_id()==0)
	{
  /// set the value of alpha metrics in the first trellis step
  ptr[0].alpha_mem[0][0] = 0;
  for (state_idx=1; state_idx<NS; state_idx++)
		{
    	ptr[0].alpha_mem[0][state_idx] = SM_INIT;
		}

  /// copy inter siso state metrics
  for(siso_idx=1; siso_idx<P; siso_idx++)
    for (state_idx=0; state_idx<NS; state_idx++)
		{
      ptr[siso_idx].alpha_mem[0][state_idx] = ptr[siso_idx-1].alpha_ext_mem[state_idx];
		}
  for(siso_idx=0; siso_idx<P-1; siso_idx++)
    for (state_idx=0; state_idx<NS; state_idx++)
		{
    	ptr[siso_idx].beta_local_mem[NWP-1][state_idx] = ptr[siso_idx+1].beta_ext_mem[state_idx];
		}

  /// copy the value of beta metrics in the last trellis step
  for (state_idx=0; state_idx<NS; state_idx++)
	{
    ptr[P-1].beta_local_mem[NWP-1][state_idx] = beta_init[state_idx];
	}
	}

	rt_team_barrier();

	for (siso_idx=0; siso_idx<P; siso_idx++)
  {
		if(rt_core_id()==siso_idx)
	{
    /// windows loop
    for (wind_idx=0; wind_idx<NWP; wind_idx++)
    {
      /// initialize beta metrics of current window
      for (state_idx=0; state_idx<NS; state_idx++)
			{
				ptr[siso_idx].beta[0][state_idx] = ptr[siso_idx].beta_local_mem[wind_idx][state_idx];
			}

      delta = siso_idx*NWP*W + wind_idx*W;
      //////// sliding window parallel BCJR algorithm
 			beta_idx = siso1_wind(flag,delta, llro + delta, llri + delta, &ptr[siso_idx],  dec + delta, ic, ic_size, W, EXT_MAX_VAL, EXT_MIN_VAL);

      /// save beta metrics for next iteration
      for (state_idx=0; state_idx<NS; state_idx++)
				ptr[siso_idx].beta_local_mem[wind_idx][state_idx] = ptr[siso_idx].beta[beta_idx][state_idx];
      
      /// initialize alpha metrics of next window
      for (state_idx=0; state_idx<NS; state_idx++)
				ptr[siso_idx].alpha_mem[0][state_idx] = ptr[siso_idx].alpha_mem[W][state_idx];

    }
  }
	}
	rt_team_barrier();

if(rt_core_id()==0)
{
  /// copy inter siso state metrics
  for(siso_idx=0; siso_idx<P; siso_idx++)
  {
    for (state_idx=0; state_idx<NS; state_idx++)
    {
      ptr[siso_idx].alpha_ext_mem[state_idx] = ptr[siso_idx].alpha_mem[W][state_idx];
      ptr[siso_idx].beta_ext_mem[state_idx] = ptr[siso_idx].beta_local_mem[0][state_idx];
    }

    /// shift the beta local memory content
    for (wind_idx=1; wind_idx<NWP; wind_idx++)
      for (state_idx=0; state_idx<NS; state_idx++)      
	ptr[siso_idx].beta_local_mem[wind_idx-1][state_idx] = ptr[siso_idx].beta_local_mem[wind_idx][state_idx];
  }
}
}
