#ifndef _DECODER_H
#define _DECODER_H


/// siso memory structure to ease parallel windowed decoding
typedef struct siso_mem {
	int **alpha_mem;      /// alpha memory
  int **beta_local_mem; /// beta local memory
  int *beta[2];         /// beta values
  int *alpha_ext_mem;   /// alpha_ext_mem values
  int *beta_ext_mem;    /// beta_ext_mem values
} siso_mem_t;
  struct allocation
	{
 int K;
 int P;//non può essere più di 7 perché un core deve essere sempre impiegato per fornire i frame. Dato che i core sono massimo 8 P deve essere sempre <=7
 int W;
 int min_iter;
 int max_iter;
 int max_frame;
 int NBF_INT;
 int EXT_MAX_VAL;
 int EXT_MIN_VAL;
 int NB_SM;
 int f_num;
 int NS;
 int NWP;
 int N;
 int flag;
 int *llr1_out;
 int *llr2_out;
 int *dec1;
 int *llr_in[2];
 siso_mem_t *siso_memory[2];
 int *beta_init[2];
	};
/// correction table computation
///\param **ic is the correction table pointer address
///\param nbit is the number of fractional bits
///\return the correction table size
int compute_table(int **ic, const int nbit); 
/// decoder side interleaver function
///\param K is the block size
///\param *perm is the permutation array
///\param *src is the array with data to permute
///\param *dst is the array with permuted data
void scrambler(int *perm, int *src, int *dst, int K);
/// decoder side deinterleaver function
///\param K is the block size
///\param *perm is the permutation array
///\param *src is the array with data to permute
///\param *dst is the array with permuted data
void descrambler(int *perm, int *src, int *dst, int K);
/// Parallel, in-order siso function
///\param NS is the number of states
///\param *llro is array of LLR exchanged by siso1 with siso2
///\param *llri is the intrinsic information array: u p
///\param SM_INIT is the state metric initialization value (-intinity approximation)
///\param P is the parallelism degree
///\param W is the window length
///\param NWP is the number of windows per siso
///\param *ptr is the array of pointers to siso memory structures
///\param EXT_MAX_VAL is the extrinsic information maximum value
///\param EXT_MIN_VAL is the extrinsic information minimum value
///\param *dec is the array with the decision information
///\param *ic is the correction table for the log-map
///\param ic_size is the correction table size
///\param *beta_init is the last trellis section initialization
void SISO1(int flag, int NS, int *llro, const int *llri, const int SM_INIT, int P,int W, int NWP,int EXT_MAX_VAL, int EXT_MIN_VAL, siso_mem_t *ptr, int *dec, int *ic, int ic_size, int *beta_init);

#endif
