#ifndef _DECODER_H
#define _DECODER_H


/// siso memory structure to ease parallel windowed decoding
typedef struct siso_mem {
	short **alpha_mem;      /// alpha memory
  short **beta_local_mem; /// beta local memory
  short *beta[2];         /// beta values
  short *alpha_ext_mem;   /// alpha_ext_mem values
  short *beta_ext_mem;    /// beta_ext_mem values
} siso_mem_t;
  struct allocation
	{
 short K;
 signed char P;//non può essere più di 7 perché un core deve essere sempre impiegato per fornire i frame. Dato che i core sono massimo 8 P deve essere sempre <=7
 signed char W;
 signed char min_iter;
 signed char max_iter;
 signed char max_frame;
 signed char NBF_INT;
 signed char EXT_MAX_VAL;
 signed char EXT_MIN_VAL;
 signed char NB_SM;
 signed char f_num;
 signed char NS;
 short NWP;
 short N;
 signed char flag;
 signed char *llr1_out;
 signed char *llr2_out;
 short *dec1;
 signed char *llr_in[2];
 siso_mem_t *siso_memory[2];
 short *beta_init[2];
	};
/// correction table computation
///\param **ic is the correction table pointer address
///\param nbit is the number of fractional bits
///\return the correction table size
int compute_table(int **ic, const int nbit); 
/// decoder side interleaver function
///\param K is the block size
///\param *perm is the permutation array
///\param *src is the array with data to permute
///\param *dst is the array with permuted data
void scrambler(short *perm, signed char *src, signed char *dst, short K);
/// decoder side deinterleaver function
///\param K is the block size
///\param *perm is the permutation array
///\param *src is the array with data to permute
///\param *dst is the array with permuted data
void descrambler(short *perm, signed char *src, signed char *dst, short K);
/// Parallel, in-order siso function
///\param NS is the number of states
///\param *llro is array of LLR exchanged by siso1 with siso2
///\param *llri is the intrinsic information array: u p
///\param SM_INIT is the state metric initialization value (-intinity approximation)
///\param P is the parallelism degree
///\param W is the window length
///\param NWP is the number of windows per siso
///\param *ptr is the array of pointers to siso memory structures
///\param EXT_MAX_VAL is the extrinsic information maximum value
///\param EXT_MIN_VAL is the extrinsic information minimum value
///\param *dec is the array with the decision information
///\param *ic is the correction table for the log-map
///\param ic_size is the correction table size
///\param *beta_init is the last trellis section initialization
void SISO1(signed char flag, signed char NS, signed char *llro, const signed char *llri, const int SM_INIT, signed char P,signed char W, short NWP,signed char EXT_MAX_VAL, signed char EXT_MIN_VAL, siso_mem_t *ptr, short *dec, int *ic, int ic_size, short *beta_init);

#endif
